import { SimpleGamesystem } from "./SimpleGamesystem";
import { ScriptAccountAction } from "./actions/ScriptAccountAction";
import { ScriptAccountCondition } from "./conditions/ScriptAccountCondition";
import { SimpleState } from "./states/SimpleState";
import { SimpleTransition } from "./transitions/SimpleTransition";

export class SimpleTemplateGamesystem extends SimpleGamesystem {
  referenceSystem: SimpleGamesystem

  constructor(referenceSystem: SimpleGamesystem) {
    super(referenceSystem.componentName, referenceSystem.componentDescription)
    this.referenceSystem = referenceSystem;

    this.referenceSystem.states.forEach(state => {
      const templateState = new SimpleState(state.stateLabel, state.stateDescription)
      templateState.initial = state.initial;
      templateState.conditions = state.conditions.map(condition => ScriptAccountCondition.constructScriptAccountCondition(condition.scriptAccount, condition.minValue, condition.maxValue)!)

      this.states.push(templateState);
    });

    this.referenceSystem.transitions.forEach(transition => {
      const startingState = this.findStateByStateLabel(transition.startingState.stateLabel);
      const endingState = this.findStateByStateLabel(transition.endingState.stateLabel);

      const templateTransition = new SimpleTransition(startingState!, endingState!)

      templateTransition.scriptAccountConditions = transition.scriptAccountConditions.map(condition =>
          ScriptAccountCondition.constructScriptAccountCondition(condition.scriptAccount, condition.minValue, condition.maxValue)!)

      templateTransition.scriptAccountActions = transition.scriptAccountActions.map(action => {
        return new ScriptAccountAction(action.scriptAccount, action.changingValue);
      });

      this.transitions.push(templateTransition);
    })


  }


}
